@@ -3,6 +3,7 @@ from copy import deepcopy
 import json
 import collections
 from abc import abstractmethod
+from shutil import copyfile
 from mcloud.container import PrebuiltImageBuilder, DockerfileImageBuilder
 from mcloud.util import Interface
 import os
@@ -66,9 +67,11 @@ class OrderedDictYAMLLoader(yaml.Loader):
             mapping[key] = value
         return mapping
 
+
+
 class YamlConfig(IConfig):
 
-    def __init__(self, file=None, source=None, app_name=None, path=None):
+    def __init__(self, file=None, source=None, app_name=None, path=None, env=None):
 
         if not file is None:
             if not os.path.exists(str(file)):
@@ -87,12 +90,32 @@ class YamlConfig(IConfig):
 
         self.config = None
 
+        self.env = env
+
     def get_services(self):
         """
         @rtype: dict[str, Service]
         """
         return self.services
 
+    def get_volumes(self):
+        """
+        @rtype: dict[str, str]
+        """
+        try:
+            return self.config['$volumes']
+        except KeyError:
+            return {}
+
+    def get_hosts(self):
+        """
+        @rtype: dict[str, str]
+        """
+        try:
+            return self.config['$hosts']
+        except KeyError:
+            return {}
+
     def get_service(self, name):
         """
         @rtype: Service
@@ -112,8 +135,7 @@ class YamlConfig(IConfig):
                 cfg = json.JSONDecoder(object_pairs_hook=collections.OrderedDict).decode(self._source)
 
             path = self.path
-
-            self.prepare(config=cfg)
+            cfg = self.prepare(config=cfg)
 
             self.validate(config=cfg)
 
@@ -131,6 +153,22 @@ class YamlConfig(IConfig):
     def export(self):
         return json.dumps(self.config)
 
+    def filter_env(self, config):
+        if not isinstance(config, dict):
+            return config
+
+        result = OrderedDict()
+        for key, val in config.items():
+            if key[0] == '~':
+                if key[1:] == self.env:
+                    if not isinstance(val, dict):
+                        raise TypeError('Child items of ~xxx elements must be dictionaries')
+                    result.update(self.filter_env(val))
+            else:
+                result[key] = self.filter_env(val)
+
+        return result
+
     def prepare(self, config):
         """
         Apply preprocessing to yaml config
@@ -139,6 +177,8 @@ class YamlConfig(IConfig):
         :return:
         """
 
+        config = self.filter_env(config)
+
         for service, cfg in config.items():
             if 'extend' in cfg:
                 new_config = deepcopy(config[cfg['extend']])
@@ -147,10 +187,12 @@ class YamlConfig(IConfig):
 
                 config[service] = new_config
 
+        return config
+
     def validate(self, config):
         try:
             Schema({
-                Required(basestring): {
+                basestring: {
                     'wait': int,
                     'image': basestring,
                     'build': basestring,
@@ -163,14 +205,38 @@ class YamlConfig(IConfig):
                         basestring: basestring
                     },
 
-                    'cmd': basestring
+                    'cmd': basestring,
+                },
+
+                '$volumes': {
+                    basestring: basestring
+                },
+
+                '$hosts': {
+                    basestring: basestring
+                },
+
+                '$events': {
+                    basestring: [basestring]
+                },
+
+                '$deploy': {
+                    basestring: [basestring]
+                },
 
-                }
             })(config)
 
-            for service in config.values():
+            has_service = False
+            for key, service in config.items():
+                if key[0] == '$':
+                    continue
                 if not 'image' in service and not 'build' in service:
                     raise ValueError('You should define "image" or "build" as a vay to build a container.')
+
+                has_service = True
+            if not has_service:
+                raise ValueError('You should define at least one service')
+
         except MultipleInvalid as e:
             raise ValueError(e)
 
@@ -225,11 +291,13 @@ class YamlConfig(IConfig):
 
     def process_env_build(self, service, config, path):
         service.env = {}
+        if self.env:
+            service.env['env'] = self.env
 
         if 'env' in config and len(config['env']):
             for name, val in config['env'].items():
                 service.env[name] = str(val)
-
+    #2dhqiyN5ig
     def process_image_build(self, service, config, path):
 
         # way to build
@@ -246,6 +314,9 @@ class YamlConfig(IConfig):
     def process(self, config, path, app_name=None):
 
         for name, service in config.items():
+            if name[0] == '$':
+                continue
+
             if app_name:
                 name = '%s.%s' % (name, app_name)
 
@@ -260,6 +331,22 @@ class YamlConfig(IConfig):
 
             # expose mcloud api
             s.volumes.append({'local': '/var/run/mcloud', 'remote': '/var/run/mcloud'})
-            s.volumes.append({'local': dirname(__file__) + '/api.py', 'remote': '/usr/bin/@me'})
+
+            # prevents monting paths with versions inside
+            # like "/usr/share/python/mcloud/lib/python2.7/site-packages/mcloud-0.7.11-py2.7.egg/mcloud/api.py"
+            if os.path.exists('/var/mcloud_api.py') or os.access('/var/', os.W_OK):
+
+                try:
+                    # copy to some constant location
+                    copyfile(dirname(__file__) + '/api.py', '/var/mcloud_api.py')
+                    # prevents write by others
+                    os.chmod('/var/mcloud_api.py', 0755)
+                    # then mount
+                    s.volumes.append({'local': '/var/mcloud_api.py', 'remote': '/usr/bin/@me'})
+                except IOError:
+                    s.volumes.append({'local': dirname(__file__) + '/api.py', 'remote': '/usr/bin/@me'})
+            else:
+                # seems to be we are in unprivileged mode (dev?), so just mount as it is
+                s.volumes.append({'local': dirname(__file__) + '/api.py', 'remote': '/usr/bin/@me'})
 
             self.services[name] = s
